# Tropical MCMC -----------------------------------------------------------------
# Rory Page
# 05/22/2019

rm(list = ls())


# Settings ----------------------------------------------------------------
library(lpSolveAPI)
library(parallel)
library(ape)
library(phangorn)
#library(maps)
library(phytools)
library(pals)

#pull in functions
setwd("C:/Users/rlpage/Desktop/Thesis/R_Help")
source("./func_ssh1.R")

#set to gene tree locations
setwd("C:/Users/rlpage/Desktop/Thesis/data")

# Enter a folder name holding the gene trees
fold_name <- ''

# Loop through the varying species depths (SD_sizes)
sdFile <- list.files(paste("./", sep=""))#,pattern = ".dat")

# TrPCA_MCMC(folder storing all tree .dat-files, cold_rate, hot_rate, number of .dat files)... very particular about make
cl <- makeCluster(8)
Genes_Total <- TrPCA_MCMC(fold_name,5,50,2)

best_trees <- read.tree(paste("./z_comb_tree_",fold_name,".dat",sep=""))

###############################
#
# Plotting for unknown species
#
###############################

trop_pca_proj_plot_known <- function(Genes_Total,best_trees,name_file){
  # Set up some parameters
  n <- length(Genes_Total[[1]]$tip.label)
  pcs <- 3
  to<- Genes_Total[[1]]$tip.label
  N <- length(Genes_Total)
  
  # Create distance measures
  distVec_all <- distMat(Genes_Total, tipOrder = to)
  D_all <- matrix(unlist(distVec_all), ncol=N)
  
  distVec_comb <- distMat(best_trees, tipOrder = to)
  N <- length(distVec_comb)
  D_comb <- matrix(unlist(distVec_comb), ncol=N)
  
  ### Tropical PCA Projections
  
  sumDist_best <- tropDist(D_comb, distVec_all)
  
  new_base <- D_comb
  
  numvectors <- dim(D_all)[2]
  
  adj_proj_points <- lapply(distVec_all,project_pi,D_s=new_base)
  
  proj_trees_matrix <-lapply(adj_proj_points,make.matrix,n=n,tips=to)
  proj_trees <- lapply(proj_trees_matrix,upgma)
  
  DD_base = t(new_base)
  D_base <- normalize.ultrametrices(DD_base)
  proj_points_plot<-lapply(adj_proj_points,polytope_iso,D=DD_base) # changed with our projected points from new tropMCMC
  
  proj_plot_norm<-lapply(proj_points_plot,normalize.proj)
  N <- length(proj_plot_norm)
  proj_2D_plot_m<-matrix(unlist(proj_plot_norm),nrow=N,ncol=3,byrow=T)
  
  
  rich8equal = c("#0081FF", "tomato3","seagreen1", "wheat4","mediumorchid1", "#FDEE02", "#FFAB00", "gainsboro","cyan","black")
  
  
  ## Tropical Triangle 2D Plot by Species Tree
  pdf(paste("./","Tropical_Triangle_",'name_file',".pdf", sep=""),width = 5, height = 10)

  par(mfrow=c(1,1))
  k <- ncol(D_base)
  plot(D_base[1,],D_base[2,])
  for(i in 1:(k - 1)){
    for(j in (i + 1):k){
      tseg1 <- tropical.geodesic.dim.2(D_base[,i],D_base[,j])
      tseg2 <- tropical.geodesic.dim.2(D_base[,i],D_base[,j],flag=1)
      if(tseg1[[2]] < tseg2[[2]]) tseg <- tseg1
      else tseg <- tseg2
      segments(tseg[[1]][1,1],tseg[[1]][2,1],tseg[[1]][1,2],tseg[[1]][2,2],col= 'black')
      segments(tseg[[1]][1,2],tseg[[1]][2,2],tseg[[1]][1,3],tseg[[1]][2,3],col= 'black')
    }
  }
  
  points(x=proj_2D_plot_m[1:1000,2],y=proj_2D_plot_m[1:1000,3],pch=16,cex=0.6,col= "red")
  points(x=proj_2D_plot_m[1001:2000,2],y=proj_2D_plot_m[1001:2000,3],pch=16,cex=0.6,col= "blue")
  
  
  #points(x=proj_2D_plot_m[1:1000,2],y=proj_2D_plot_m[1:1000,3],pch=16,cex=0.4,col= 'blue')
  #points(x=proj_2D_plot_m[1001:2000,2],y=proj_2D_plot_m[1001:2000,3],pch=16,cex=0.4,col= 'red')
  #points(x=proj_2D_plot_m[2001:3000,2],y=proj_2D_plot_m[2001:3000,3],pch=16,cex=0.4,col= rich8equal[9])
  #points(x=proj_2D_plot_m[3001:4000,2],y=proj_2D_plot_m[3001:4000,3],pch=16,cex=0.4,col= rich8equal[4])
  #points(x=proj_2D_plot_m[4001:5000,2],y=proj_2D_plot_m[4001:5000,3],pch=16,cex=0.4,col= rich8equal[3])
  #points(x=proj_2D_plot_m[5001:6000,2],y=proj_2D_plot_m[5001:6000,3],pch=16,cex=0.4,col= rich8equal[6])
  #points(x=proj_2D_plot_m[6001:7000,2],y=proj_2D_plot_m[6001:7000,3],pch=16,cex=0.4,col= rich8equal[7])
  #points(x=proj_2D_plot_m[7001:8000,2],y=proj_2D_plot_m[7001:8000,3],pch=16,cex=0.4,col= rich8equal[8])
  
  dev.off()
}

# Remember to change colors and lengths of plotting depending on your trees
cl <- makeCluster(8)
trop_pca_proj_plot_known(GT,comb_trees,'test_plot')


###############################
#
# Plotting for unknown species
#
###############################

cl <- makeCluster(8)

# Set up some parameters
n <- length(Genes_Total[[1]]$tip.label)
pcs <- 3
to<- Genes_Total[[1]]$tip.label
N <- length(Genes_Total)

# Create distance measures
distVec_all <- distMat(Genes_Total, tipOrder = to)
D_all <- matrix(unlist(distVec_all), ncol=N)

distVec_comb <- distMat(best_trees, tipOrder = to)
N <- length(distVec_comb)
D_comb <- matrix(unlist(distVec_comb), ncol=N)

### Tropical PCA Projections

sumDist_best <- tropDist(D_comb, distVec_all)

new_base <- D_comb

numvectors <- dim(D_all)[2]

adj_proj_points <- lapply(distVec_all,project_pi,D_s=new_base)

proj_trees_matrix <-lapply(adj_proj_points,make.matrix,n=n,tips=to)
proj_trees <- lapply(proj_trees_matrix,upgma)

DD_base = t(new_base)
D_base <- normalize.ultrametrices(DD_base)
proj_points_plot<-lapply(adj_proj_points,polytope_iso,D=DD_base) # changed with our projected points from new tropMCMC

proj_plot_norm<-lapply(proj_points_plot,normalize.proj)
N <- length(proj_plot_norm)
proj_2D_plot_m<-matrix(unlist(proj_plot_norm),nrow=N,ncol=3,byrow=T)



###################################################
#
#   Measuring distances and clustering by distances
#
##################################################
cl <- makeCluster(8)
type <- seq(numvectors)
for(i in seq(numvectors)){
  if(i==type[i]) {
    for(j in seq(i, numvectors)) {
      if(RF.dist(proj_trees[[i]],proj_trees[[j]])==0) {
        type[j]=i
      }
    } 
  }
}


table(type)

# Decreasing matrix of grouped tree topologies!
type_ind <- as.matrix(sort(table(type), decreasing = T))

tree_num <- as.numeric(rownames(type_ind))

tree_num

#### Plot the topologies grouped by distances to each other
####

rich8equal = c("#0081FF", "tomato3","seagreen1", "wheat4","mediumorchid1", "#FDEE02", "#FFAB00", "gainsboro","cyan","black")

pdf("test_plot.pdf",width = 5, height = 10)
par(mfrow=c(3,3))
for(i in 1:length(tree_num)){
  if (i < 9){
    plot(proj_trees[[tree_num[i]]], main=paste("(",type_ind[i,],")"),col.main=rich8equal[i])
  }else{
    plot(proj_trees[[tree_num[i]]], main=paste("(",type_ind[i,],")"),col.main="black")
  }
}

dev.off()

freq <- type

freq_list <- list(dimnames(sort(table(type), decreasing = T)))

# Number of different topologies
length(tree_num)


0.95*length(Genes_Total)

#Change the number to ==  95% above.... sorry it's manually changed
sum(sort(table(type), decreasing = T)[1:46])

as.numeric(freq_list[[1]]$type[1])


pal <- function(col, border = "light gray", ...){
  n <- length(col)
  plot(0, 0, type="n", xlim = c(0, 1), ylim = c(0, 1),
       axes = FALSE, xlab = "", ylab = "", ...)
  rect(0:(n-1)/n, 0, 1:n/n, 1, col = col, border = border)
}


rich8equal = c("#0081FF", "tomato3","seagreen1", "wheat4","mediumorchid1", 
               "#FDEE02", "#FFAB00", "gainsboro","cyan","black")
pal(rich8equal)


rich12equal = c("#000040", "#000093", "#0020E9", "#0076FF", "#00B8C2", 
                "#04E466", "#49FB25", "#E7FD09", "#FEEA02", "#FFC200", 
                "#FF8500", "#FF3300")
pal(rich12equal)



tol21rainbow= c("#771155", "#AA4488", "#CC99BB", "#114477", "#4477AA", 
                "#77AADD", "#117777", "#44AAAA", "#77CCCC", "#117744", 
                "#44AA77", "#88CCAA", "#777711", "#AAAA44", "#DDDD77", 
                "#774411", "#AA7744", "#DDAA77", "#771122", "#AA4455", 
                "#DD7788")

pal(tol21rainbow)


tol58rainbow= c("#771155", "#AA4488", "#CC99BB", "#114477", "#4477AA", 
                "#77AADD", "#117777", "#44AAAA", "#77CCCC", "#117744", 
                "#44AA77", "#88CCAA", "#777711", "#AAAA44", "#DDDD77", 
                "#774411", "#AA7744", "#DDAA77", "#771122", "#AA4455", 
                "#DD7788", "#000040", "#000093", "#0020E9", "#0076FF", 
                "#00B8C2", "#04E466", "#49FB25", "#E7FD09", "#FEEA02", 
                "#FFC200", "#FF8500", "#FF3300", 'forestgreen', 'red2',
                'orange', 'cornflowerblue','magenta', 'darkolivegreen4',
                'indianred1', 'darkblue','mediumorchid1','firebrick4',
                'yellowgreen', 'lightsalmon','darkgray', 'wheat4', 
                '#DDAD4B', 'seagreen1','moccasin', 
                'mediumvioletred', 'seagreen','cadetblue1',
                "darkolivegreen1", "tomato3" , "#7CE3D8","gainsboro")

pal(tol58rainbow)

pal.bands(polychrome(n=36))

# x number of Species Types
for(i in 1:46){
  if(type[i]==as.numeric(freq_list[[1]]$type[1])) freq[i]<- rich8equal[1] else freq[i] <- "black"
  if(type[i]==as.numeric(freq_list[[1]]$type[2])) freq[i]<- rich8equal[2]
  if(type[i]==as.numeric(freq_list[[1]]$type[3])) freq[i]<- rich8equal[3]
  if(type[i]==as.numeric(freq_list[[1]]$type[4])) freq[i]<- rich8equal[4]
  if(type[i]==as.numeric(freq_list[[1]]$type[5])) freq[i]<- rich8equal[5]
  if(type[i]==as.numeric(freq_list[[1]]$type[6])) freq[i]<- rich8equal[6]
  if(type[i]==as.numeric(freq_list[[1]]$type[7])) freq[i]<- rich8equal[7]
  if(type[i]==as.numeric(freq_list[[1]]$type[8])) freq[i]<- rich8equal[8]
  if(type[i]==as.numeric(freq_list[[1]]$type[9])) freq[i]<- rich8equal[9]
  if(type[i]==as.numeric(freq_list[[1]]$type[10])) freq[i]<- tol58rainbow[10]
  if(type[i]==as.numeric(freq_list[[1]]$type[11])) freq[i]<- tol58rainbow[11]
  if(type[i]==as.numeric(freq_list[[1]]$type[12])) freq[i]<- tol58rainbow[12]
  if(type[i]==as.numeric(freq_list[[1]]$type[13])) freq[i]<- tol58rainbow[13]
  if(type[i]==as.numeric(freq_list[[1]]$type[14])) freq[i]<- tol58rainbow[14]
  if(type[i]==as.numeric(freq_list[[1]]$type[15])) freq[i]<- tol58rainbow[15]
  if(type[i]==as.numeric(freq_list[[1]]$type[16])) freq[i]<- tol58rainbow[16]
  if(type[i]==as.numeric(freq_list[[1]]$type[17])) freq[i]<- tol58rainbow[17]
  if(type[i]==as.numeric(freq_list[[1]]$type[18])) freq[i]<- tol58rainbow[18]
  if(type[i]==as.numeric(freq_list[[1]]$type[19])) freq[i]<- tol58rainbow[19]
  if(type[i]==as.numeric(freq_list[[1]]$type[20])) freq[i]<- tol58rainbow[20]
  if(type[i]==as.numeric(freq_list[[1]]$type[21])) freq[i]<- tol58rainbow[21]
  if(type[i]==as.numeric(freq_list[[1]]$type[22])) freq[i]<- tol58rainbow[22]
  if(type[i]==as.numeric(freq_list[[1]]$type[23])) freq[i]<- tol58rainbow[23]
  if(type[i]==as.numeric(freq_list[[1]]$type[24])) freq[i]<- tol58rainbow[24]
  if(type[i]==as.numeric(freq_list[[1]]$type[25])) freq[i]<- tol58rainbow[25]
  if(type[i]==as.numeric(freq_list[[1]]$type[26])) freq[i]<- tol58rainbow[26]
  if(type[i]==as.numeric(freq_list[[1]]$type[27])) freq[i]<- tol58rainbow[27]
  if(type[i]==as.numeric(freq_list[[1]]$type[28])) freq[i]<- tol58rainbow[28]
  if(type[i]==as.numeric(freq_list[[1]]$type[29])) freq[i]<- tol58rainbow[29]
  if(type[i]==as.numeric(freq_list[[1]]$type[30])) freq[i]<- tol58rainbow[30]
  if(type[i]==as.numeric(freq_list[[1]]$type[31])) freq[i]<- tol58rainbow[31]
  if(type[i]==as.numeric(freq_list[[1]]$type[32])) freq[i]<- tol58rainbow[32]
  if(type[i]==as.numeric(freq_list[[1]]$type[33])) freq[i]<- tol58rainbow[33]
  if(type[i]==as.numeric(freq_list[[1]]$type[34])) freq[i]<- tol58rainbow[34]
  if(type[i]==as.numeric(freq_list[[1]]$type[35])) freq[i]<- tol58rainbow[35]
  if(type[i]==as.numeric(freq_list[[1]]$type[36])) freq[i]<- tol58rainbow[36]
  if(type[i]==as.numeric(freq_list[[1]]$type[37])) freq[i]<- tol58rainbow[37]
  if(type[i]==as.numeric(freq_list[[1]]$type[38])) freq[i]<- tol58rainbow[38]
  if(type[i]==as.numeric(freq_list[[1]]$type[39])) freq[i]<- tol58rainbow[39]
  if(type[i]==as.numeric(freq_list[[1]]$type[40])) freq[i]<- tol58rainbow[40]
  if(type[i]==as.numeric(freq_list[[1]]$type[41])) freq[i]<- tol58rainbow[41]
  if(type[i]==as.numeric(freq_list[[1]]$type[42])) freq[i]<- tol58rainbow[42]
  if(type[i]==as.numeric(freq_list[[1]]$type[43])) freq[i]<- tol58rainbow[43]
  if(type[i]==as.numeric(freq_list[[1]]$type[44])) freq[i]<- tol58rainbow[44]
  if(type[i]==as.numeric(freq_list[[1]]$type[45])) freq[i]<- tol58rainbow[45]
  if(type[i]==as.numeric(freq_list[[1]]$type[46])) freq[i]<- tol58rainbow[46]
  if(type[i]==as.numeric(freq_list[[1]]$type[47])) freq[i]<- tol58rainbow[47]
  if(type[i]==as.numeric(freq_list[[1]]$type[48])) freq[i]<- tol58rainbow[48]
  if(type[i]==as.numeric(freq_list[[1]]$type[49])) freq[i]<- tol58rainbow[49]
  if(type[i]==as.numeric(freq_list[[1]]$type[50])) freq[i]<- tol58rainbow[50]
  if(type[i]==as.numeric(freq_list[[1]]$type[51])) freq[i]<- tol58rainbow[1]
  if(type[i]==as.numeric(freq_list[[1]]$type[52])) freq[i]<- tol58rainbow[2]
  #if(type[i]==as.numeric(freq_list[[1]]$type[53])) freq[i]<- tol58rainbow[3]
  #if(type[i]==as.numeric(freq_list[[1]]$type[54])) freq[i]<- tol58rainbow[4]
  #if(type[i]==as.numeric(freq_list[[1]]$type[55])) freq[i]<- tol58rainbow[5]
  #if(type[i]==as.numeric(freq_list[[1]]$type[56])) freq[i]<- tol58rainbow[6]
  #if(type[i]==as.numeric(freq_list[[1]]$type[57])) freq[i]<- tol58rainbow[7]
  #if(type[i]==as.numeric(freq_list[[1]]$type[58])) freq[i]<- tol58rainbow[8]
  #if(type[i]==as.numeric(freq_list[[1]]$type[59])) freq[i]<- tol58rainbow[9]
  #if(type[i]==as.numeric(freq_list[[1]]$type[60])) freq[i]<- tol58rainbow[10]
  #if(type[i]==as.numeric(freq_list[[1]]$type[61])) freq[i]<- tol58rainbow[11]
  #if(type[i]==as.numeric(freq_list[[1]]$type[62])) freq[i]<- tol58rainbow[12]
  #if(type[i]==as.numeric(freq_list[[1]]$type[63])) freq[i]<- tol58rainbow[13]
  #if(type[i]==as.numeric(freq_list[[1]]$type[64])) freq[i]<- tol58rainbow[14]
  #if(type[i]==as.numeric(freq_list[[1]]$type[65])) freq[i]<- tol58rainbow[15]
  #if(type[i]==as.numeric(freq_list[[1]]$type[66])) freq[i]<- tol58rainbow[16]
  #if(type[i]==as.numeric(freq_list[[1]]$type[67])) freq[i]<- tol58rainbow[17]
  #if(type[i]==as.numeric(freq_list[[1]]$type[68])) freq[i]<- tol58rainbow[18]
  #if(type[i]==as.numeric(freq_list[[1]]$type[69])) freq[i]<- tol58rainbow[19]
  #if(type[i]==as.numeric(freq_list[[1]]$type[70])) freq[i]<- tol58rainbow[20]
  #if(type[i]==as.numeric(freq_list[[1]]$type[71])) freq[i]<- tol58rainbow[21]
  #if(type[i]==as.numeric(freq_list[[1]]$type[72])) freq[i]<- tol58rainbow[22]
  #if(type[i]==as.numeric(freq_list[[1]]$type[73])) freq[i]<- tol58rainbow[23]
  #if(type[i]==as.numeric(freq_list[[1]]$type[74])) freq[i]<- tol58rainbow[24]
  #if(type[i]==as.numeric(freq_list[[1]]$type[75])) freq[i]<- tol58rainbow[25]
  #if(type[i]==as.numeric(freq_list[[1]]$type[76])) freq[i]<- tol58rainbow[26]
  #if(type[i]==as.numeric(freq_list[[1]]$type[77])) freq[i]<- tol58rainbow[27]
  #if(type[i]==as.numeric(freq_list[[1]]$type[78])) freq[i]<- tol58rainbow[28]
  #if(type[i]==as.numeric(freq_list[[1]]$type[79])) freq[i]<- tol58rainbow[29]
  #if(type[i]==as.numeric(freq_list[[1]]$type[80])) freq[i]<- tol58rainbow[30]
  #if(type[i]==as.numeric(freq_list[[1]]$type[81])) freq[i]<- tol58rainbow[31]
  #if(type[i]==as.numeric(freq_list[[1]]$type[82])) freq[i]<- tol58rainbow[32]
  #if(type[i]==as.numeric(freq_list[[1]]$type[83])) freq[i]<- tol58rainbow[33]
  #if(type[i]==as.numeric(freq_list[[1]]$type[84])) freq[i]<- tol58rainbow[34]
  #if(type[i]==as.numeric(freq_list[[1]]$type[85])) freq[i]<- tol58rainbow[35]
  #if(type[i]==as.numeric(freq_list[[1]]$type[86])) freq[i]<- tol58rainbow[36]
  #if(type[i]==as.numeric(freq_list[[1]]$type[87])) freq[i]<- tol58rainbow[37]
  #if(type[i]==as.numeric(freq_list[[1]]$type[88])) freq[i]<- tol58rainbow[38]
  #if(type[i]==as.numeric(freq_list[[1]]$type[89])) freq[i]<- tol58rainbow[39]
}



pdf("Tropical_Triangle_proj_test.pdf")
plot.trop.triangle(D_base,freq)
dev.off()

